//  Xerver Free Web Server
//  Copyright (C) 2002-2007 Omid Rouhani
//
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//
//  #############################################################
//  ##  YOU CAN CONTACT THE AUTHOR (OMID ROUHANI) AT:          ##
//  ##  HTTP://WWW.JAVASCRIPT.NU/XERVER/                       ##
//  ##                                                         ##
//  ##  IF YOUR SOFTWARE IS NOT RELEASED UNDER THE             ##
//  ##  GNU GENERAL PUBLIC LICENSE (GPL),                      ##
//  ##  PLEASE DO NOT COPY ANYTHING FROM THIS SOURCE CODE!!!   ##
//  ##                                                         ##
//  ##  FOR FULL LICENSE, PLEASE READ "XERVER LICENSE".        ##
//  #############################################################



package ftp_server;
import webserver.*;
import common.*;
import ftp_server.*;
import java.util.*;
import java.io.*;




/**
 *
 * <B>About this class:</B>
 * <BR>
 * This class represents a database with all user accounts.
 *
 * @author <a href="http://www.JavaScript.nu/xerver/" TARGET="_top">Omid Rouhani</a>
 * @version 1.0
 */



final public class UserDatabase
{
	final static private boolean b_showErrors=false;
	private final static String s_userFolder=FTPServerController.s_userFolder;
	private final static String s_userDataExtension=FTPServerController.s_userDataExtension;
	private String [] as_allUsers;
	private int i_numberOfUsers;
	private HashMap hm_dataBase;

	public UserDatabase()
	{
		updateDatabase();
	}


	/**
	* Update the information stored in this class from file (in case an user data file has been changed).
	*/
	public void updateDatabase()
	{
			try {
			File f_userFolder=new File(s_userFolder);
			as_allUsers=f_userFolder.list(new DatabaseFileFilter());	//Only get files with ".dat"
			i_numberOfUsers=as_allUsers.length;
			//"If the initial capacity is greater than the maximum number of entries divided by the load factor, no rehash operations will ever occur."
			//The line above says that if "argument below" > maxEntries/0.75 ==> No (expensive) rehash() will occur, so we set [ "argument below"=initial capacity >> as_allUsers.length() ]
			hm_dataBase=new HashMap(i_numberOfUsers*2+2);//new TreeMap();

			//Note HashMap is NOT synchronized, for this reason...
			//If we want add a new entry at any moment to the database,
			//create a completely new UserDatabase (and a new HashMap),
			//instead of make just an "put()" to add a new value.
			//Because if we make an "put()" and a "get()" at the same time
			//we don't know what we end up with.
			//Createing a new UserDatabase is indeed costly but it will only
			//occur when admin change settings to the server and it's
			//a safe way to make changes to this user-database.


			String tmpUsername;//, tmpFilename;
			for (int i=0; i<as_allUsers.length; i++)
			{
	//			tmpFilename=as_allUsers[i];
	//			if (tmpFilename.endsWith(s_userDataExtension))
	//			{
					as_allUsers[i]=as_allUsers[i].substring(0,as_allUsers[i].length()-4);
					hm_dataBase.put(as_allUsers[i],new UserData(as_allUsers[i]));
	//			}
			}
		}
		catch (Exception e)
		{
			if (b_showErrors)
			{
				System.out.println("An error occured @ updateDatabase: "+e);
				e.printStackTrace();
			}
		}
	}


	/**
	* Get the user data for the user <CODE>s_username</CODE>, represented as a <CODE>UserData</CODE>.
	*/
	public UserData getUserData(String s_username)
	{
		return (UserData)hm_dataBase.get(s_username);
	}

	/**
	* Get all account names stored in this database.
	*/
	public String [] getAllUsers()
	{
		return as_allUsers;
	}
}